# https://hpiers.obspm.fr/eop-pc/models/constants.html#rotation

# https://en.wikipedia.org/wiki/Sidereal_time#Sidereal_day
T_SIDEREAL_DAY = 86164.0905

# https://en.wikipedia.org/wiki/Standard_gravitational_parameter
GM_EARTH = 3.986004418e14

# https://en.wikipedia.org/wiki/Earth%27s_rotation#
EARTH_ROTATION_RATE = 7.2921150E-05

# https://en.wikipedia.org/wiki/Earth_radius
EARTH_EQUATORIAL_RADIUS = 6378137

# https://en.wikipedia.org/wiki/Geopotential_spherical_harmonic_model
J2 = 1.0826359e-3

# 地球椭球扁率
# https://en.wikipedia.org/wiki/Latitude#Latitude_on_the_ellipsoid
INVERSE_FLATTENING = 298.257223563

# https://en.wikipedia.org/wiki/Astronomical_unit
AU = 149_597_870_700  # 天文单位，米
